/*
 ============================================================================
 Name		 : DynamicArrays.cpp
 Copyright   : Symbian Academy, 2007
 Description : Exercise: Dynamic Arrays
 ============================================================================
 */

//  Include Files  

#include <e32base.h>
#include <e32std.h>
#include <e32cons.h>			// Console

// Function prototype (C-style).
LOCAL_C void MainL();
GLDEF_C TInt E32Main();

//  Global Variables
LOCAL_D CConsoleBase* console;

//  Student-Class
_LIT(KStudent, "%S (%d), %d\n");
class TStudent
	{
public:
	TStudent(const TInt aNumber, const TDesC& aName, const TInt aAge);
	void ToStringL(TDes& aString);
	// Edit 10a: Declare a comparison function that should compare the name
	//           of two students.


	// Edit 12a: Declare a matching function that should match both the name
	//           and the number, but not the age of two students.


public:
	/** Name of the student */
	TBuf<20> iName;
	/** Matriculation number */
	TInt iNumber;
	/** Age */
	TInt iAge;
	};

TStudent::TStudent(const TInt aNumber, const TDesC& aName, const TInt aAge) :
	iNumber(aNumber), iAge(aAge)
	{
	// Copy the name from the parameter to our own buffer
	iName.Copy (aName);
	}

// Edit 10b: Implement the comparision function that compares the name of two elements.
//           Hint: You can use the CompareF-function of TDesC.



// Edit 12b: Implement the matching function that should match both the name
//           and the number, but not the age of two students.
//           This function should make it possible to search for a student based on its
//           name and number, but without specifying the age, as this will change over time.



//  Local Functions

/**
 * Fills the string with the objects properties.
 * The string will not be allocated by this function.
 * An approximate length-check will cause a KErrOverflow-leave if the
 * string does not have the required length. The check is not exact,
 * so there might also be caused a panic, depending on the contents.
 * @param aString string to fill with information.
 */
void TStudent::ToStringL(TDes& aString)
	{
	// Approximate length check, should be made more exact.
	if ( aString.MaxLength ()< iName.Length ()+ 15)
		User::Leave (KErrOverflow);
	aString.Format (KStudent, &iName, iAge, iNumber);
	}

/**
 * Prints the students array to the console in a formatted way.
 * @param aArray the students array to print.
 */
void PrintStudentArrayL(RArray<TStudent>& aArray)
	{
	_LIT(KArrayElement, "#%d: ");
	// Create a modifiable heap buffer which will contain the
	// output lines one after another.
	RBuf tmpBuf;
	tmpBuf.CleanupClosePushL ();
	tmpBuf.CreateL (40);
	for (int i = 0; i < aArray.Count (); i++)
		{
		// Output the current array position
		console->Printf (KArrayElement, i);
		// Let the array element write itself to our RBuf and then output it to the console.
		aArray[i].ToStringL (tmpBuf);
		console->Printf (tmpBuf);
		}
	CleanupStack::PopAndDestroy (1);
	}

LOCAL_C void MainL()
	{
	//
	// add your program code here, example code below
	//

	// ------------------------------------------------------------------
	// TFixedArray
	// ------------------------------------------------------------------
	// Edit 1: Create a TFixedArray called "studentNumbers" and use it to
	//         store four TInt-values.


	
	
	

	// ------------------------------------------------------------------
	// Creating and filling the array
	// ------------------------------------------------------------------
	// Edit 2: Create an RArray that can hold four TStudent-objects.
	//         Choose an appropriate granularity and create a key
	//         based on TStudent.iNumber
	
	// Edit 3: Put the array on the cleanup stack, ensuring that it
	//         is correctly closed in case of a leave.
	//         Also do Edit 3a at the end of this function to destroy 
	//         the object again!
	

	// Edit 4: Create four student-objects (s1 to s4) using the numbers
	//         from the studentNumbers-array you declared.
	//         Declare four names using _LIT-macros. Choose an age
	//         for each student.


	
	
	
	
	
	
	

	// Edit 5: Append three (!) of the four students to the array.


	

	console->Printf(_L("\nOriginal array\n"));
	// PrintStudentArrayL(students);

	// ------------------------------------------------------------------
	// Sorting the array
	// ------------------------------------------------------------------
	console->Printf(_L("\nSorted array\n"));
	// Edit 6: Sort the array based on the key with the appropriate sorting
	//         function (signed / unsigned)
	
	
	// PrintStudentArrayL(students);

	// ------------------------------------------------------------------
	// Ordered Insert
	// ------------------------------------------------------------------
	console->Printf(_L("\nOrdered Insert\n"));
	// Edit 7: Insert the 4th student in the appropriate key order.
	
	
	//PrintStudentArrayL(students);

	// ------------------------------------------------------------------
	// Finding an element
	// ------------------------------------------------------------------
	console->Printf(_L("\nFinding an element\n"));
	// Edit 8: Adapt the "findNum"-variable to a number that is contained
	//         in your array.
	const TInt findNum = 742;
	TInt index = KErrNotFound;
	// Edit 9: Use the correct function for doing a binary search for the
	//         number specified by findNum.
	

	if (index == KErrNotFound)
		{
		_LIT(KFindError, "Did not find number %d\n");
		console->Printf(KFindError, findNum);
		}
	else
		{
		_LIT(KFindSuccess, "Student %d: %S\n");
		//console->Printf(KFindSuccess, findNum, &students[index].iName);
		}

	// ------------------------------------------------------------------
	// Sorting using a custom comparison function
	// ------------------------------------------------------------------
	console->Printf(_L("\nSort by name\n"));
	// Edit 10: Do edits 10a and 10b above (extend the student class with
	//          a comparison function)
	// Edit 11: Sort the array based on your custom comparison function.


	
	//PrintStudentArrayL(students);

	// ------------------------------------------------------------------
	// Finding using a custom match function
	// ------------------------------------------------------------------
	// Edit 12: Do edits 12a and 12b above (extend the student class with
	//          a comparison function)
	
	// Edit 13: Search for one of your students using his name and number,
	//          based on your custom match function.
	TInt idx = KErrNotFound;
	

	console->Printf(_L("\nSearch student using name and number:\n"));
	if (idx == KErrNotFound)
		{
		console->Printf(_L("No student was found\n"));
		}
	else
		{
		_LIT(KFindSuccess, "Found student: %S\n");
		//console->Printf(KFindSuccess, &students[idx].iName);
		}

	// ------------------------------------------------------------------
	// Cleanup
	// ------------------------------------------------------------------
	// Edit 3a: Destroy the array using the cleanup stack.
	

	// ------------------------------------------------------------------
	// To train your knowledge of descriptors and memory handling,
	// rewrite the TStudent-class to a CStudent-class, which
	// saves the name in an RBuf (instead of a TBuf with fixed length).
	// Rewrite the code in the MainL()-function to use a RPointerArray
	// instead of an RArray to correctly handle heap-based CStudent-objects.
	// See the tutorial document of this module for further instructions.
	}

// ------------------------------------------------------------------
// Main Application
// ------------------------------------------------------------------
GLDEF_C TInt E32Main()
	{
	// Create cleanup stack
	CTrapCleanup* cleanup = CTrapCleanup::New ();

	// Create output console
	_LIT(KTextConsoleTitle, "Console - Dynamic Arrays");
	TRAPD(createError, console = Console::NewL(KTextConsoleTitle, TSize(KConsFullScreen,KConsFullScreen)))
	;
	if ( createError)
		return createError;

	// Run application code inside TRAP harness
	__UHEAP_MARK;

	TRAPD(mainError, MainL())
	;
	_LIT(KTextFailed, "MainL() failed, leave code = %d\n");
	if ( mainError)
		{
		console->Printf (KTextFailed, mainError);
		}

	// Wait for the space key before exiting.
	// Generally better than accepting any key to prevent immediate shutdown
	// if a key has still been in the queue when the app. started.
	_LIT(KTextPressSpace, "\n[Press space to exit]\n");
	console->Printf (KTextPressSpace);
	while (console->Getch ()!= EKeySpace)
		{
		}

	// Check the state of the heap memory - if the same amount of memory
	// is available as before MainL() was executed, everything's fine.
	// Otherwise, a panic will be raised, which would result in a graphical
	// error dialog in an UI framework, but exits the application when
	// using the console mode.
	__UHEAP_MARKEND;

	// If the program passed the __UHEAP_MARKEND-macro, no memory
	// leaks were detected
	_LIT(KTextMemoryLeak, "\nNo memory leaks detected! [Press space]\n");
	console->Printf (KTextMemoryLeak);
	while (console->Getch ()!= EKeySpace)
		{
		}

	// Cleanup
	delete console;
	delete cleanup;
	return KErrNone;
	}

